/*
     This file is part of GNUnet.
     (C) 2001, 2002, 2003 Christian Grothoff (and other contributing authors)

     GNUnet is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2, or (at your
     option) any later version.

     GNUnet is distributed in the hope that it will be useful, but
     WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
     General Public License for more details.

     You should have received a copy of the GNU General Public License
     along with GNUnet; see the file COPYING.  If not, write to the
     Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
*/

/**
 * @file util/checksum.c
 * @brief various helper methods
 * @author Christian Grothoff
 **/ 

#include "gnunet_util.h"

/**
 * Compute the CRC32 checksum for the first len bytes of the buffer.
 *
 * @param buf the data over which we're taking the CRC
 * @param len the length of the buffer
 * @return the resulting CRC32 checksum
 **/
int crc32N(void * buf, int len) {
  uLong crc;
  crc = crc32(0L, Z_NULL, 0);
  crc = crc32(crc, buf, len);
  return crc;
}

/**
 * This method does not really belong here, but where else to put
 * it...
 *
 * @return a random value in the interval [0,i[. 
 **/
int randomi(int i) {
  int ret;

  if (i <= 0)
    errexit("FATAL: randomi() called with %d as the argument\n", 
	    i); 

  ret = rand() % i;

  if ( (ret < 0) || 
       (ret >= i) )
    errexit("FATAL: randomi broken (%d, %d)\n",
	    i, 
	    ret);  
  return ret;
}

/**
 * Get an array with a random permutation of the numbers 0...n-1.
 **/
int * permute(int n) {
  int * ret;
  int i;
  int tmp;
  int x;    

  ret = MALLOC(n * sizeof(int));
  for (i=0;i<n;i++)
    ret[i] = i;
  for (i=0;i<n;i++) {
    x = randomi(n);
    tmp = ret[x];
    ret[x] = ret[i];
    ret[i] = tmp;
  }
  return ret;
}


/* **************** endian conversion helpers ************* */

/**
 * This method does not really belong here, but where else to put
 * it...
 **/
unsigned long long ntohll(unsigned long long n) {
#if __BYTE_ORDER == __BIG_ENDIAN
  return n; 
#else
  return (((unsigned long long)ntohl(n)) << 32) + ntohl(n >> 32);
#endif
}

/**
 * This method does not really belong here, but where else to put
 * it...
 **/
unsigned long long htonll(unsigned long long n) {
#if __BYTE_ORDER == __BIG_ENDIAN
  return n; 
#else
  return (((unsigned long long)htonl(n)) << 32) + htonl(n >> 32);
#endif
}

/* end of checksum.c */
